﻿using System;
using System.Collections.Generic;
using System.Linq;
using Android.Content;
using Android.Graphics;
using Android.Views;
using AndroidX.Core.Content;
using AndroidX.RecyclerView.Widget;

namespace AuthenticatorPro.List
{
    internal class SheetMenuAdapter : RecyclerView.Adapter
    {
        private readonly Context _context;
        private readonly List<SheetMenuItem> _items;
        
        public event EventHandler ItemClick;
        public override int ItemCount => _items.Count;
        
        public SheetMenuAdapter(Context context, List<SheetMenuItem> items)
        {
            _context = context;
            _items = items;
        }
        
        public override void OnBindViewHolder(RecyclerView.ViewHolder viewHolder, int position)
        {
            var item = _items.ElementAtOrDefault(position);

            if(item == null)
                return;

            var holder = (SheetMenuItemViewHolder) viewHolder;
            holder.ItemView.Click += (sender, args) =>
            {
                item.Handler.Invoke(sender, args);
                ItemClick?.Invoke(sender, args);
            };
            
            holder.Icon.SetImageResource(item.Icon);
            holder.Title.SetText(item.Title);

            if(item.Description != null)
            {
                holder.Description.Visibility = ViewStates.Visible;
                holder.Description.SetText(item.Description.Value);
            }

            if(item.IsSensitive)
            {
                var colourValue = ContextCompat.GetColor(_context, Resource.Color.colorError);
                var colour = Color.Rgb(Color.GetRedComponent(colourValue), Color.GetBlueComponent(colourValue), Color.GetGreenComponent(colourValue));
                    
                holder.Icon.SetColorFilter(colour);
                holder.Title.SetTextColor(colour);
            }
        }

        public override RecyclerView.ViewHolder OnCreateViewHolder(ViewGroup parent, int viewType)
        {
            var itemView = LayoutInflater.From(parent.Context).Inflate(Resource.Layout.sheetMenuItem, parent, false);
            return new SheetMenuItemViewHolder(itemView);
        }
    }
}